package nl.amis.rest.controller;

import java.io.IOException;

import java.io.StringReader;

import java.util.ArrayList;
import java.util.Collection;

import javax.servlet.jsp.JspContext;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import nl.amis.rest.resources.DeptResource;
import nl.amis.rest.resources.DeptsResource;
import nl.amis.rest.resources.EmpsResource;
import nl.amis.rest.resources.Thing;
import nl.amis.rest.resources.ThingResource;
import nl.amis.rest.resources.ThingsResource;

import oracle.adf.model.BindingContainer;
import oracle.adf.model.binding.DCBindingContainer;
import oracle.adf.model.binding.DCIteratorBinding;
import oracle.adf.model.servlet.HttpBindingContext;
import oracle.adf.share.ADFContext;
import oracle.adf.share.el.OracleExpressionEvaluatorImpl;

import oracle.jbo.Row;
import oracle.jbo.RowIterator;
import oracle.jbo.uicli.binding.JUCtrlRangeBinding;
import oracle.jbo.uicli.binding.JUIteratorBinding;


import oracle.xml.parser.v2.XMLDocument;
import oracle.xml.parser.v2.XSLException;
import oracle.xml.parser.v2.XSLProcessor;
import oracle.xml.parser.v2.XSLStylesheet;

import org.restlet.Application;
import org.restlet.Restlet;
import org.restlet.Router;
import org.restlet.data.MediaType;
import org.restlet.resource.DomRepresentation;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


public class RestfulApplication extends Application {

    /**
     * Creates a root Restlet that will receive all incoming calls.
     */
    @Override
    public synchronized Restlet createRoot() {
        // Create a router Restlet that defines routes.
        Router router = new Router(getContext());

        // Defines a route for the resource "list of departments"
        router.attach("/depts", DeptsResource.class);
        // Defines a route for the resource "department)"
        router.attach("/depts/{deptno}", DeptResource.class);
        // Defines a route for the resource "list of employees (within current department)"
        router.attach("/depts/{deptno}/emps", EmpsResource.class);
        // Defines a route for the resource "list of employees (within current department)"
        router.attach("/emps", EmpsResource.class);

        return router;
    }

    public static DCBindingContainer getHrmService() {
        ADFContext ctx = ADFContext.getCurrent();
        OracleExpressionEvaluatorImpl elev =
            (OracleExpressionEvaluatorImpl)ctx.getExpressionEvaluator();
        HttpBindingContext data = (HttpBindingContext)elev.evaluate("${data}");
        DCBindingContainer bc =
            data.findBindingContainer("myfirstrestfulservice_HrmServicePageDef");
        // the servlet filter took care of preparing the binding container
        return bc;
    }


    public static DomRepresentation createDomRepresentationForRows(String rootElement,
                                                                   RowIterator ri,
                                                                   int maxRows,
                                                                   int startRow,
                                                                   boolean singleRow,
                                                                   XSLStylesheet xsl) throws IOException {
        DomRepresentation representation =
            new DomRepresentation(MediaType.TEXT_XML);
        Document d = representation.getDocument();
        Element r = d.createElement(rootElement);
        d.appendChild(r);
        Row[] rows;
        if (singleRow) { // indicates a single row representation
            rows = new Row[] { ri.getCurrentRow() };
        } else {
            ri.reset();
            ri.setRangeSize(maxRows);
            ri.setRangeStart(startRow);
            rows = ri.getAllRowsInRange();
        }

        for (Row row : rows) {
            Node node = null;
            if (xsl != null)
                node = row.writeXML(0, row.XML_OPT_LIMIT_RANGE, xsl);
            else
                node = row.writeXML(0, row.XML_OPT_LIMIT_RANGE);

            Node importedRow = d.importNode(node, true);
            r.appendChild(importedRow);
        }

        d.normalizeDocument();
        return representation;
    }

    public static JUCtrlRangeBinding getThingsTableBinding() {
        ADFContext ctx = ADFContext.getCurrent();
        OracleExpressionEvaluatorImpl elev =
            (OracleExpressionEvaluatorImpl)ctx.getExpressionEvaluator();
        HttpBindingContext data = (HttpBindingContext)elev.evaluate("${data}");
        DCBindingContainer bc =
            data.findBindingContainer("myfirstrestfulservice_untitled1PageDef");
        bc.refresh(DCBindingContainer.PREPARE_MODEL);
        JUCtrlRangeBinding tableBinding =
            (JUCtrlRangeBinding)bc.findNamedObject("Thing");
        return tableBinding;
    }

    public void setThings(Collection<Thing> things) {
        //   this.things = things;
    }

    public Collection<Thing> getThings() {

        return null;
    }

    public static XSLStylesheet getXslFromString(String xslcontent) {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        // Turn on validation, and turn off namespaces
        factory.setValidating(true);
        factory.setNamespaceAware(false);
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
        }
        InputSource is = new InputSource(new StringReader(xslcontent));
        Document doc = null;
        try {
            doc = builder.parse(is);
        } catch (SAXException e) {
            System.out.println(e.getMessage());
        } catch (IOException e) {
        }
        XSLStylesheet xsl = null;
        try {
            xsl = new XSLProcessor().newXSLStylesheet((XMLDocument)doc);
        } catch (XSLException e) {
        }
        return xsl;
    }


}
